<?php

namespace Bixo\Whatsapp\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Whatsapp\Forms\Chat as ChatForm;
use Bixo\Whatsapp\Http\Requests\ChatResourceRequest;
use Bixo\Whatsapp\Http\Resources\ChatResource;
use Bixo\Whatsapp\Http\Resources\ChatsCollection;
use Bixo\Whatsapp\Models\Chat;
use Bixo\Whatsapp\Actions\ChatAction;
use Bixo\Whatsapp\Actions\ChatActions;


/**
 * Resource controller class for chat.
 */
class ChatResourceController extends BaseController
{

    /**
     * Initialize chat resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = ChatForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.whatsapp.modules'), 'whatsapp', guard_url('whatsapp'));
            return $next($request);
        });
    }

    /**
     * Display a list of chat.
     *
     * @return Response
     */
    public function index(ChatResourceRequest $request)
    {
        $request = $request->all();
        $page = ChatActions::run('paginate', $request);

        $data = new ChatsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('whatsapp::chat.names'))
            ->view('whatsapp::chat.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display chat.
     *
     * @param Request $request
     * @param Model   $chat
     *
     * @return Response
     */
    public function show(ChatResourceRequest $request, Chat $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ChatResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('whatsapp::chat.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('whatsapp::chat.show')
            ->output();
    }

    /**
     * Show the form for creating a new chat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ChatResourceRequest $request, Chat $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ChatResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('whatsapp::chat.name'))
            ->view('whatsapp::chat.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new chat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ChatResourceRequest $request, Chat $model)
    {
        try {
            $request = $request->all();
            $model = ChatAction::run('store', $model, $request);
            $data = new ChatResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('whatsapp::chat.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('whatsapp/chat/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/whatsapp/chat'))
                ->redirect();
        }

    }

    /**
     * Show chat for editing.
     *
     * @param Request $request
     * @param Model   $chat
     *
     * @return Response
     */
    public function edit(ChatResourceRequest $request, Chat $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ChatResource($model);
        // return view('whatsapp::chat.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('whatsapp::chat.name'))
            ->view('whatsapp::chat.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the chat.
     *
     * @param Request $request
     * @param Model   $chat
     *
     * @return Response
     */
    public function update(ChatResourceRequest $request, Chat $model)
    {
        try {
            $request = $request->all();
            $model = ChatAction::run('update', $model, $request);
            $data = new ChatResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('whatsapp::chat.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('whatsapp/chat/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('whatsapp/chat/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the chat.
     *
     * @param Model   $chat
     *
     * @return Response
     */
    public function destroy(ChatResourceRequest $request, Chat $model)
    {
        try {

            $request = $request->all();
            $model = ChatAction::run('destroy', $model, $request);
            $data = new ChatResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('whatsapp::chat.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('whatsapp/chat/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('whatsapp/chat/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
